@@ -17,8 +17,9 @@ static void log_bad_env_cmd(int linenum)
     log(loglevel_t::ERROR, "Unknown command in environment file (line ", linenum, ")");
 }
 
-// Read and set environment variables from a file. May throw std::bad_alloc, std::system_error.
-void read_env_file(const char *env_file_path, bool log_warnings, environment &env, bool throw_on_open_failure)
+void read_env_file(const char *env_file_path, int resolve_fd, bool log_warnings, environment &env,
+        bool throw_on_open_failure)
 {
-    read_env_file_inline(env_file_path, log_warnings, env, throw_on_open_failure, log_bad_env, log_bad_env_cmd);
+    read_env_file_inline(env_file_path, resolve_fd, log_warnings, env, throw_on_open_failure,
+            log_bad_env, log_bad_env_cmd);
 }
