@@ -5,9 +5,14 @@
 #include <sys/types.h>
 #include <sys/resource.h>
 
+#if SUPPORT_CAPABILITIES
+#include <sys/capability.h>
+#endif
+
 #include <baseproc-sys.h>
 #include <service.h>
 #include <dinit-utmp.h>
+#include <dinit-util.h>
 
 // This header defines base_proc_service (base process service) and several derivatives, as well as some
 // utility functions and classes. See service.h for full details of services.
@@ -31,9 +36,23 @@ struct run_proc_params
     #if SUPPORT_CGROUPS
     const char *run_in_cgroup = nullptr; //  cgroup path
     #endif
+    #if SUPPORT_CAPABILITIES
+    cap_iab_t cap_iab = nullptr;
+    unsigned long secbits = 0;
+    bool no_new_privs = false;
+    #endif
     bool on_console;          // whether to run on console
     bool in_foreground;       // if on console: whether to run in foreground
     bool unmask_sigint = false; // if in foreground: whether to unmask SIGINT
+    bool nice_is_set = false;
+    int nice = 0;             // the process nice value
+    #if SUPPORT_IOPRIO
+    int ioprio = -1;          // scheduling class and priority for the process
+    #endif
+    #if SUPPORT_OOM_ADJ
+    bool oom_adj_is_set = false;
+    short oom_adj = 0;        // oom score adjustment value
+    #endif
     int wpipefd;              // pipe to which error status will be sent (if error occurs)
     int csfd;                 // control socket fd (or -1); may be moved
     int socket_fd;            // pre-opened socket fd (or -1); may be moved
@@ -186,8 +205,26 @@ class base_process_service : public service_record
     unsigned log_buf_size = 0; // log buffer current size
     std::vector<char, default_init_allocator<char>> log_buffer;
 
+    bool nice_is_set = false;
+    int nice;
+
+#if SUPPORT_IOPRIO
+    int ioprio = -1;
+#endif
+
+#if SUPPORT_OOM_ADJ
+    bool oom_adj_is_set = false;
+    short oom_adj = 0;
+#endif
+
     std::vector<service_rlimits> rlimits; // resource limits
 
+#if SUPPORT_CAPABILITIES
+    cap_iab_wrapper cap_iab;
+    unsigned long secbits = 0;
+    bool no_new_privs = false;
+#endif
+
 #if SUPPORT_CGROUPS
     string run_in_cgroup;
 #endif
@@ -459,6 +496,35 @@ class base_process_service : public service_record
     }
     #endif
 
+    #if SUPPORT_CAPABILITIES
+    void set_cap(cap_iab_wrapper &&iab, unsigned int sbits) noexcept
+    {
+        cap_iab = std::move(iab);
+        secbits = sbits;
+    }
+    #endif
+
+    void set_nice(int nice_v, bool is_set) noexcept
+    {
+        nice_is_set = is_set;
+        nice = nice_v;
+    }
+
+    #if SUPPORT_IOPRIO
+    void set_ioprio(int ioprio_v) noexcept
+    {
+        ioprio = ioprio_v;
+    }
+    #endif
+
+    #if SUPPORT_OOM_ADJ
+    void set_oom_adj(short oom_adj_v, bool is_set) noexcept
+    {
+        oom_adj_is_set = is_set;
+        oom_adj = oom_adj_v;
+    }
+    #endif
+
     void set_rlimits(std::vector<service_rlimits> &&rlimits_p)
     {
         rlimits = std::move(rlimits_p);
@@ -732,6 +798,7 @@ class process_service : public base_process_service
 class bgproc_service : public process_service
 {
     virtual void handle_exit_status() noexcept override;
+    virtual void exec_succeeded() noexcept override;
     virtual void exec_failed(run_proc_err errcode) noexcept override;
 
     enum class pid_result_t {
