@@ -36,13 +36,14 @@ enum class service_event_t {
 };
 
 /* Shutdown types */
-enum class shutdown_type_t {
+enum class shutdown_type_t : char {
     NONE,              // No explicit shutdown
     REMAIN,            // Continue running with no services
     HALT,              // Halt system without powering down
     POWEROFF,          // Power off system
     REBOOT,            // Reboot system
-    SOFTREBOOT         // Reboot dinit
+    SOFTREBOOT,        // Reboot dinit
+    KEXEC              // Reboot with kexec (without firmware reinitialisation)
 };
 
 /* Reasons for why service stopped */
@@ -71,9 +72,10 @@ enum class exec_stage {
     ARRANGE_FDS, READ_ENV_FILE, SET_NOTIFYFD_VAR, SETUP_ACTIVATION_SOCKET, SETUP_CONTROL_SOCKET,
     CHDIR, SETUP_STDINOUTERR, ENTER_CGROUP, SET_RLIMITS, SET_UIDGID,
     OPEN_LOGFILE, // this is used instead of SETUP_STDINOUTERR if output is to logfile
+    SET_CAPS, SET_PRIO,
 
     /* values for future expansion: */
-    SPARE2, SPARE3, SPARE4, SPARE5, SPARE6, SPARE7, SPARE8,
+    SPARE4, SPARE5, SPARE6, SPARE7, SPARE8,
     /* must be last: */ DO_EXEC
 };
 
