@@ -20,6 +20,7 @@
 #include "dinit-log.h"
 #include "dinit-util.h"
 #include "dinit-utmp.h"
+#include "dinit-iostream.h"
 
 using string = std::string;
 using string_iterator = std::string::iterator;
@@ -35,7 +36,9 @@ static void do_env_subst(const char *setting_name, ha_string &line,
 {
     using namespace dinit_load;
     std::string line_s = std::string(line.c_str(), line.length());
-    value_var_subst(setting_name, line_s, offsets, resolve_env_var, &envmap, arg);
+    value_var_subst(setting_name, line_s, offsets, [&](const std::string &name) {
+        return resolve_env_var(name, envmap);
+    }, arg);
     line = line_s;
 }
 
@@ -392,12 +395,13 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
     service_record *rval = nullptr;
     service_record *dummy = nullptr;
 
-    ifstream service_file;
+    dio::istream service_file;
     string service_filename;
 
     int fail_load_errno = 0;
     std::string fail_load_path;
     const char *service_dsc_dir = nullptr;
+    std::pair<int,int> sdf_fds;
 
     // Couldn't find one. Have to load it.
     for (auto &service_dir : service_dirs) {
@@ -408,16 +412,18 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
         }
         service_filename += name;
 
-        service_file.open(service_filename.c_str(), ios::in);
-        if (service_file) break;
+        sdf_fds = open_with_dir(service_dsc_dir, name.c_str());
+        if (sdf_fds.first != -1) {
+            break; // found
+        }
 
-        if (errno != ENOENT && fail_load_errno == 0) {
-            fail_load_errno = errno;
+        if (sdf_fds.second != ENOENT && fail_load_errno == 0) {
+            fail_load_errno = sdf_fds.second;
             fail_load_path = std::move(service_filename);
         }
     }
 
-    if (!service_file) {
+    if (sdf_fds.first == -1) {
         if (fail_load_errno == 0) {
             throw service_not_found(name);
         }
@@ -426,13 +432,18 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
         }
     }
 
+    fd_holder sdf_parent_fd = sdf_fds.first;
+
+    service_file.set_fd(sdf_fds.second);
+    service_file.check_buf();
+
     service_settings_wrapper<prelim_dep> settings;
     service_record *consumer_of_svc = nullptr;
 
     string line;
     // getline can set failbit if it reaches end-of-file, we don't want an exception in that case. There's
     // no good way to handle an I/O error however, so we'll have exceptions thrown on badbit:
-    service_file.exceptions(ios::badbit);
+    //service_file.exceptions(ios::badbit);
 
     bool create_new_record = true;
 
@@ -494,45 +505,55 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
     }
 
     file_input_stack input_stack;
-    input_stack.push(std::move(service_filename), std::move(service_file));
+    input_stack.push(std::move(service_filename), std::move(service_file), sdf_parent_fd.release());
 
     try {
+        environment srv_env;
+
+        // Build an environment mapping which for now just contains variables from the main
+        // environment (including that inherited by the process). This will be used for pre-load
+        // substitution (eg inside @include paths, in dependency names).
+        environment::env_map srv_envmap = main_env.build();
+
+        auto resolve_var = [&](const std::string &name) {
+            return srv_envmap.lookup(name);
+        };
+
         process_service_file(name, input_stack,
                 [&](string &line, file_pos_ref fpr, string &setting, setting_op_t op,
                         string_iterator &i, string_iterator &end) -> void {
 
-            auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist,
-                    const std::string &waitsford, dependency_type dep_type) -> void {
-                const string &service_filename = input_stack.current_file_name();
-                process_dep_dir(*this, name.c_str(), service_filename, deplist, waitsford,
-                        dep_type, reload_svc);
-            };
+                    auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist,
+                            const std::string &waitsford, dependency_type dep_type) -> void {
+                        const string &service_filename = input_stack.current_file_name();
+                        process_dep_dir(*this, name.c_str(), service_filename, deplist, waitsford,
+                                dep_type, reload_svc);
+                    };
 
-            auto load_service_n = [&](const string &dep_name) -> service_record * {
-                try {
-                    return load_service(dep_name.c_str(), reload_svc);
-                }
-                catch (service_description_exc &sle) {
-                    log_service_load_failure(sle);
-                    throw service_load_exc(name, "could not load dependency.");
-                }
-                catch (service_load_exc &sle) {
-                    log(loglevel_t::ERROR, "Could not load service ", sle.service_name, ": ",
-                            sle.exc_description);
-                    throw service_load_exc(name, "could not load dependency.");
-                }
-            };
+                    auto load_service_n = [&](const string &dep_name) -> service_record * {
+                        try {
+                            return load_service(dep_name.c_str(), reload_svc);
+                        }
+                        catch (service_description_exc &sle) {
+                            log_service_load_failure(sle);
+                            throw service_load_exc(name, "could not load dependency.");
+                        }
+                        catch (service_load_exc &sle) {
+                            log(loglevel_t::ERROR, "Could not load service ", sle.service_name,
+                                    ": ", sle.exc_description);
+                            throw service_load_exc(name, "could not load dependency.");
+                        }
+                    };
 
-            process_service_line(settings, name.c_str(), argval, line, fpr, setting,
-                    op, i, end, load_service_n, process_dep_dir_n);
-        });
+                    process_service_line(settings, name.c_str(), argval, line, fpr, setting,
+                            op, i, end, load_service_n, process_dep_dir_n, resolve_var);
+                },
+                argval, resolve_var);
 
         auto report_err = [&](const char *msg){
             throw service_load_exc(name, msg);
         };
 
-        environment srv_env;
-
         // Fill user vars before reading env file
         if (settings.export_passwd_vars) {
             fill_environment_userinfo(settings.run_as_uid, name, srv_env);
@@ -545,26 +566,18 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
             srv_env.set_var(std::move(envname));
         }
 
-        // This mapping is temporary, for load substitutions. (The environment actually *may* change
-        // after load, e.g. through dinitctl setenv, either from the outside or from within services,
-        // and so we need to calculate a fresh mapping on each process invocation).
-        environment::env_map srv_envmap;
-
         if (!settings.env_file.empty()) {
+            fd_holder env_resolve_fd = std::move(settings.env_file_dir_fd);
             try {
-                if (settings.env_file[0] == '/') {
-                    // (don't allocate a string if we don't have to)
-                    read_env_file(settings.env_file.c_str(), false, srv_env, true);
-                }
-                else {
-                    std::string fullpath = combine_paths(service_dsc_dir, settings.env_file.c_str());
-                    read_env_file(fullpath.c_str(), false, srv_env, true);
-                }
+                read_env_file(settings.env_file.c_str(), env_resolve_fd.get(), false, srv_env, true);
             } catch (const std::system_error &se) {
                 throw service_load_exc(name, std::string("could not load environment file: ") + se.what());
             }
         }
 
+        // This mapping is temporary, for load substitutions. (The environment actually *may* change
+        // after load, e.g. through dinitctl setenv, either from the outside or from within services,
+        // and so we need to calculate a fresh mapping on each process invocation).
         srv_envmap = srv_env.build(main_env);
 
         settings.finalise<true>(report_err, srv_envmap, argval);
@@ -735,6 +748,16 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
             #if SUPPORT_CGROUPS
             rvalps->set_cgroup(std::move(settings.run_in_cgroup));
             #endif
+            #if SUPPORT_CAPABILITIES
+            rvalps->set_cap(std::move(settings.capabilities), settings.secbits.get());
+            #endif
+            rvalps->set_nice(settings.nice, settings.nice_is_set);
+            #if SUPPORT_IOPRIO
+            rvalps->set_ioprio(settings.ioprio);
+            #endif
+            #if SUPPORT_OOM_ADJ
+            rvalps->set_oom_adj(settings.oom_adj, settings.oom_adj_is_set);
+            #endif
             rvalps->set_rlimits(std::move(settings.rlimits));
             rvalps->set_restart_interval(settings.restart_interval, settings.max_restarts);
             rvalps->set_restart_delay(settings.restart_delay);
@@ -778,6 +801,16 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
             #if SUPPORT_CGROUPS
             rvalps->set_cgroup(std::move(settings.run_in_cgroup));
             #endif
+            #if SUPPORT_CAPABILITIES
+            rvalps->set_cap(std::move(settings.capabilities), settings.secbits.get());
+            #endif
+            rvalps->set_nice(settings.nice, settings.nice_is_set);
+            #if SUPPORT_IOPRIO
+            rvalps->set_ioprio(settings.ioprio);
+            #endif
+            #if SUPPORT_OOM_ADJ
+            rvalps->set_oom_adj(settings.oom_adj, settings.oom_adj_is_set);
+            #endif
             rvalps->set_rlimits(std::move(settings.rlimits));
             rvalps->set_pid_file(std::move(settings.pid_file));
             rvalps->set_restart_interval(settings.restart_interval, settings.max_restarts);
@@ -817,6 +850,16 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
             #if SUPPORT_CGROUPS
             rvalps->set_cgroup(std::move(settings.run_in_cgroup));
             #endif
+            #if SUPPORT_CAPABILITIES
+            rvalps->set_cap(std::move(settings.capabilities), settings.secbits.get());
+            #endif
+            rvalps->set_nice(settings.nice, settings.nice_is_set);
+            #if SUPPORT_IOPRIO
+            rvalps->set_ioprio(settings.ioprio);
+            #endif
+            #if SUPPORT_OOM_ADJ
+            rvalps->set_oom_adj(settings.oom_adj, settings.oom_adj_is_set);
+            #endif
             rvalps->set_rlimits(std::move(settings.rlimits));
             rvalps->set_stop_timeout(settings.stop_timeout);
             rvalps->set_start_timeout(settings.start_timeout);
@@ -979,11 +1022,18 @@ service_record * dirload_service_set::load_reload_service(const char *fullname,
     }
     catch (std::system_error &sys_err)
     {
+        // TODO: remove this handling when we read all files via dinit-io
         exception_cleanup();
         // don't use sys_err.what() since libstdc++ sometimes includes class names (basic_filebuf):
         std::string msg = input_stack.current_file_name() + ": " + sys_err.code().message();
         throw service_load_exc(name, std::move(msg));
     }
+    catch (dio::iostream_system_err &sys_err)
+    {
+        exception_cleanup();
+        std::string msg = input_stack.current_file_name() + ": " + strerror(sys_err.get_errno());
+        throw service_load_exc(name, std::move(msg));
+    }
     catch (std::length_error &len_err) {
         // This is pretty much only theoretically possible; we'd normally expect bad_alloc instead.
         exception_cleanup();
