@@ -54,7 +54,7 @@ void service_dir_opt::build_paths(bool am_system_init)
             const char * user_home = get_user_home();
             if (user_home != nullptr && *user_home != '\0') {
                 size_t user_home_len = strlen(user_home);
-                if (xdg_config_home != nullptr
+                if (xdg_config_home != nullptr && *xdg_config_home != '\0'
                         && xdg_config_home_len == (user_home_len + strlen("/.config"))
                         && strncmp(xdg_config_home, user_home, user_home_len) == 0
                         && strcmp(xdg_config_home + user_home_len, "/.config") == 0) {
@@ -76,7 +76,12 @@ void service_dir_opt::build_paths(bool am_system_init)
 
         done_user_home:
 
-        if (!home_service_dir_set) {
+        if (home_service_dir_set) {
+            service_dir_paths.emplace_back("/etc/dinit.d/user", /*dyn_allocd=*/false);
+            service_dir_paths.emplace_back("/usr/lib/dinit.d/user", /*dyn_allocd=*/false);
+            service_dir_paths.emplace_back("/usr/local/lib/dinit.d/user", /*dyn_allocd=*/false);
+        }
+        else {
             service_dir_paths.emplace_back("/etc/dinit.d", /*dyn_allocd=*/false);
             service_dir_paths.emplace_back("/run/dinit.d", false);
             service_dir_paths.emplace_back("/usr/local/lib/dinit.d", false);
