@@ -44,10 +44,6 @@ std::vector<const char *> separate_args(ha_string &s,
 
 void process_service::exec_succeeded() noexcept
 {
-    if (get_type() != service_type_t::PROCESS) {
-        return;
-    }
-
     tracking_child = true;
 
     // This could be a smooth recovery (state already STARTED). No need to do anything here in
@@ -83,6 +79,12 @@ void process_service::exec_succeeded() noexcept
     }
 }
 
+void bgproc_service::exec_succeeded() noexcept
+{
+    // For bgproc service, unlike regular process service, successful execution doesn't mean much;
+    // the process has to terminate (after forking) before we consider it started.
+}
+
 void scripted_service::exec_succeeded() noexcept
 {
     // For a scripted service, this means nothing other than that the start/stop
@@ -932,9 +934,23 @@ bool process_service::start_stop_process(const std::vector<const char *> &cmd) n
         run_params.force_notify_fd = -1;
         run_params.notify_var = nullptr;
         run_params.env_file = env_file.c_str();
+        run_params.nice_is_set = nice_is_set;
+        run_params.nice = nice;
         #if SUPPORT_CGROUPS
         run_params.run_in_cgroup = run_in_cgroup.c_str();
         #endif
+        #if SUPPORT_CAPABILITIES
+        run_params.cap_iab = cap_iab.get();
+        run_params.secbits = secbits;
+        run_params.no_new_privs = onstart_flags.no_new_privs;
+        #endif
+        #if SUPPORT_IOPRIO
+        run_params.ioprio = ioprio;
+        #endif
+        #if SUPPORT_OOM_ADJ
+        run_params.oom_adj_is_set = oom_adj_is_set;
+        run_params.oom_adj = oom_adj;
+        #endif
         run_child_proc(run_params);
     }
     else {
