@@ -88,7 +88,7 @@ void service_record::stopped() noexcept
     bool will_restart = desired_state == service_state_t::STARTED && !pinned_stopped;
 
     // If we won't restart, break soft dependencies now
-    if (! will_restart) {
+    if (!will_restart) {
         for (auto dept : dependents) {
             if (!dept->is_hard()) {
                 // waits-for or soft dependency:
@@ -135,7 +135,7 @@ void service_record::stopped() noexcept
     }
 
     // Start failure will have been logged already, only log if we are stopped for other reasons:
-    if (! start_failed) {
+    if (!start_failed) {
         log_service_stopped(service_name);
 
         // If this service chains to another, start the chained service now, if:
@@ -143,8 +143,9 @@ void service_record::stopped() noexcept
         // - ... successfully (i.e. exit code 0)
         // - this service won't restart, and
         // - a shutdown isn't in progress
-        if ((onstart_flags.always_chain || (did_finish(stop_reason) && get_exit_status() == 0 && ! will_restart))
-                && ! start_on_completion.empty() && ! services->is_shutting_down()) {
+        if ((onstart_flags.always_chain || (did_finish(stop_reason)
+                && get_exit_status().did_exit_clean() && !will_restart))
+                && !start_on_completion.empty() && !services->is_shutting_down()) {
             try {
                 auto chain_to = services->load_service(start_on_completion.c_str());
                 chain_to->start();
@@ -652,7 +653,26 @@ void service_record::do_stop(bool with_restart) noexcept
     in_user_restart = false;
 
     // Will we restart? desired state of STOPPED inhibits auto-restart
-    bool for_restart = with_restart || (auto_restart && desired_state == service_state_t::STARTED);
+    bool do_auto_restart = false;
+    if (auto_restart == auto_restart_mode::ALWAYS) {
+        do_auto_restart = true;
+    }
+    else if (auto_restart == auto_restart_mode::ON_FAILURE) {
+        if (get_type() == service_type_t::PROCESS || get_type() == service_type_t::BGPROCESS) {
+            auto exit_status = get_exit_status();
+            if (exit_status.was_signalled()) {
+                if (!value(exit_status.get_signal()).is_in(SIGHUP, SIGINT, SIGUSR1, SIGUSR2, SIGTERM)) {
+                    do_auto_restart = true;
+                }
+            }
+            else {
+                if (exit_status.get_exit_status() != 0) {
+                    do_auto_restart = true;
+                }
+            }
+        }
+    }
+    bool for_restart = with_restart || (do_auto_restart && desired_state == service_state_t::STARTED);
     bool restart_deps = with_restart;
 
     if (!with_restart && for_restart) {
