@@ -2,6 +2,8 @@
 #include <cstdio>
 #include <csignal>
 #include <cstring>
+#include <cstdlib>
+
 #include <string>
 #include <iostream>
 #include <exception>
@@ -246,6 +248,53 @@ class subproc_buffer : private cpbuffer<subproc_bufsize>
     }
 };
 
+#if defined(RB_POWER_OFF) || defined(RB_POWEROFF) || defined(RB_POWER_DOWN)
+#define POWER_OFF_SUPPORTED 1
+#define POWER_OFF_IS_DEFAULT 1
+#else
+#define POWER_OFF_SUPPORTED 0
+#define POWER_OFF_IS_DEFAULT 0
+#endif
+
+#if defined(RB_HALT_SYSTEM) || defined(RB_HALT)
+#define HALT_SUPPORTED 1
+#if !POWER_OFF_IS_DEFAULT
+#define HALT_IS_DEFAULT 1
+#else
+#define HALT_IS_DEFAULT 0
+#endif
+#else
+#define HALT_SUPPORTED 0
+#define HALT_IS_DEFAULT 0
+#endif
+
+#if POWER_OFF_IS_DEFAULT || HALT_IS_DEFAULT
+#define REBOOT_IS_DEFAULT 0
+#else
+#define REBOOT_IS_DEFAULT 1
+#endif
+
+static bool reboot_cmd_unsupported(const shutdown_type_t type)
+{
+    // weed out unsupported values
+    switch (type) {
+#if !HALT_SUPPORTED
+    case shutdown_type_t::HALT:
+        return true;
+#endif
+#if !POWER_OFF_SUPPORTED
+    case shutdown_type_t::POWEROFF:
+        return true;
+#endif
+#ifndef RB_KEXEC
+    case shutdown_type_t::KEXEC:
+        return true;
+#endif
+    default:
+        return false;
+    }
+}
+
 
 int main(int argc, char **argv)
 {
@@ -255,7 +304,13 @@ int main(int argc, char **argv)
     bool sys_shutdown = false;
     bool use_passed_cfd = false;
     
-    auto shutdown_type = shutdown_type_t::POWEROFF;
+#if POWER_OFF_IS_DEFAULT
+    shutdown_type_t shutdown_type = shutdown_type_t::POWEROFF;
+#elif HALT_IS_DEFAULT
+    shutdown_type_t shutdown_type = shutdown_type_t::HALT;
+#else
+    shutdown_type_t shutdown_type = shutdown_type_t::REBOOT;
+#endif
 
     const char *execname = base_name(argv[0]);
     if (strcmp(execname, reboot_execname) == 0) {
@@ -287,6 +342,9 @@ int main(int argc, char **argv)
             else if (strcmp(argv[i], "-s") == 0) {
                 shutdown_type = shutdown_type_t::SOFTREBOOT;
             }
+            else if (strcmp(argv[i], "-k") == 0) {
+                shutdown_type = shutdown_type_t::KEXEC;
+            }
             else if (strcmp(argv[i], "--use-passed-cfd") == 0) {
                 use_passed_cfd = true;
             }
@@ -304,21 +362,44 @@ int main(int argc, char **argv)
     if (show_help) {
         cout << execname << " :   shutdown the system\n"
                 "  --help           : show this help\n"
-                "  -r               : reboot\n"
-                "  -s               : soft-reboot (restart dinit with same boot-time arguments)\n"
-                "  -h               : halt system\n"
-                "  -p               : power down (default)\n"
+#if REBOOT_IS_DEFAULT
+                "  -r               : reboot (default).\n"
+#else
+                "  -r               : reboot.\n"
+#endif
+                "  -s               : soft-reboot (restart dinit with same boot-time arguments).\n"
+#if HALT_SUPPORTED && HALT_IS_DEFAULT
+                "  -h               : halt system (default).\n"
+#elif HALT_SUPPORTED
+                "  -h               : halt system.\n"
+#else
+                "  -h               : halt (not supported on this system).\n"
+#endif
+#if POWER_OFF_SUPPORTED
+                "  -p               : power down (default).\n"
+#else
+                "  -p               : power down (not supported on this system).\n"
+#endif
+#if defined(RB_KEXEC)
+                "  -k               : execute kernel loaded via kexec.\n"
+#else
+                "  -k               : execute new kernel (not supported on this system).\n"
+#endif
                 "  --use-passed-cfd : use the socket file descriptor identified by the DINIT_CS_FD\n"
                 "                     environment variable to communicate with the init daemon.\n"
                 "  --system         : perform shutdown immediately, instead of issuing shutdown\n"
-                "                     command to the init program. Not recommended for use\n"
-                "                     by users.\n";
+                "                     command to the init program. Not recommended for normal use.\n";
         return 1;
     }
     
     if (sys_shutdown) {
         do_system_shutdown(shutdown_type);
-        return 0;
+        return 1; // likely to cause panic; the above shouldn't return
+    }
+
+    if (reboot_cmd_unsupported(shutdown_type)) {
+        cerr << "Unsupported shutdown type\n";
+        return 1;
     }
 
     signal(SIGPIPE, SIG_IGN);
@@ -412,22 +493,47 @@ void do_system_shutdown(shutdown_type_t shutdown_type)
     sigfillset(&allsigs);
     sigprocmask(SIG_SETMASK, &allsigs, nullptr);
     
+    bool soft_reboot = false;
     int reboot_type = RB_AUTOBOOT; // reboot
+    const char *shutdown_type_arg = "reboot";
 #if defined(RB_POWER_OFF)
-    if (shutdown_type == shutdown_type_t::POWEROFF) reboot_type = RB_POWER_OFF;
+    if (shutdown_type == shutdown_type_t::POWEROFF) {
+        reboot_type = RB_POWER_OFF;
+        shutdown_type_arg = "poweroff";
+    }
 #elif defined(RB_POWEROFF)
     // FreeBSD spells it slightly differently
-    if (shutdown_type == shutdown_type_t::POWEROFF) reboot_type = RB_POWEROFF;
+    if (shutdown_type == shutdown_type_t::POWEROFF) {
+        reboot_type = RB_POWEROFF;
+        shutdown_type_arg = "poweroff";
+    }
 #elif defined(RB_POWERDOWN)
     // NetBSD (at least) uses RB_POWERDOWN rather than RB_POWER_OFF
-    if (shutdown_type == shutdown_type_t::POWEROFF) reboot_type = RB_POWERDOWN;
+    if (shutdown_type == shutdown_type_t::POWEROFF) {
+        reboot_type = RB_POWERDOWN;
+        shutdown_type_arg = "poweroff";
+    }
 #endif
 #if defined(RB_HALT_SYSTEM)
-    if (shutdown_type == shutdown_type_t::HALT) reboot_type = RB_HALT_SYSTEM;
+    // Linux
+    if (shutdown_type == shutdown_type_t::HALT) {
+        reboot_type = RB_HALT_SYSTEM;
+        shutdown_type_arg = "halt";
+    }
 #elif defined(RB_HALT)
     // NetBSD, FreeBSD
-    if (shutdown_type == shutdown_type_t::HALT) reboot_type = RB_HALT;
+    if (shutdown_type == shutdown_type_t::HALT) {
+        reboot_type = RB_HALT;
+        shutdown_type_arg = "halt";
+    }
 #endif
+#if defined(RB_KEXEC)
+    if (shutdown_type == shutdown_type_t::KEXEC) reboot_type = RB_KEXEC;
+#endif
+    if (shutdown_type == shutdown_type_t::SOFTREBOOT) {
+        shutdown_type_arg = "soft";
+        soft_reboot = true;
+    }
     
     // Write to console rather than any terminal, since we lose the terminal it seems:
     int consfd = open("/dev/console", O_WRONLY);
@@ -475,7 +581,7 @@ void do_system_shutdown(shutdown_type_t shutdown_type)
         int stat_r = lstat(hook_paths[i], &statbuf);
         if (stat_r == 0 && (statbuf.st_mode & execmask) != 0) {
             sub_buf.append("Executing shutdown hook...\n");
-            const char *prog_args[] = { hook_paths[i], nullptr };
+            const char *prog_args[] = { hook_paths[i], shutdown_type_arg, nullptr };
             try {
                 auto r = run_process(prog_args, loop, sub_buf);
                 if (r.did_exit() && r.get_exit_status() == 0) {
@@ -501,12 +607,31 @@ void do_system_shutdown(shutdown_type_t shutdown_type)
 
     sync();
     
+    if (soft_reboot) {
+        // For a soft reboot we are not PID 1; init is still running, and waiting for us.
+        exit(0);
+    }
+
     sub_buf.append("Issuing shutdown via kernel...\n");
     loop.poll();  // give message a chance to get to console
 #ifdef __NetBSD__
     reboot(reboot_type, NULL);
 #else
-    reboot(reboot_type);
+    if (reboot(reboot_type)) {
+        // we're in trouble now
+        sub_buf.append("reboot: ");
+        sub_buf.append(strerror(errno));
+        if (shutdown_type == shutdown_type_t::KEXEC) {
+            sub_buf.append(
+                    "\nIt is possible that no suitable kernel image was loaded before"
+                    "\nreboot with kexec was attempted.\n"
+            );
+        }
+        else {
+            sub_buf.append("\n");
+        }
+        while (true) loop.run();
+    }
 #endif
 }
 
@@ -648,10 +773,12 @@ static loop_t::child_proc_watcher::proc_status_t run_process(const char * prog_a
 static void unmount_disks(loop_t &loop, subproc_buffer &sub_buf)
 {
     try {
-#ifdef __NetBSD__
+#if __NetBSD__
         const char * unmount_args[] = { "/sbin/umount", "-a", nullptr };
-#else
+#elif __linux__
         const char * unmount_args[] = { "/bin/umount", "-a", "-r", nullptr };
+#else
+#error "Unmount command not known for this system"
 #endif
         run_process(unmount_args, loop, sub_buf);
     }
@@ -665,10 +792,12 @@ static void unmount_disks(loop_t &loop, subproc_buffer &sub_buf)
 static void swap_off(loop_t &loop, subproc_buffer &sub_buf)
 {
     try {
-#ifdef __NetBSD__
+#if __NetBSD__
         const char * swapoff_args[] = { "/sbin/swapctl", "-U", nullptr };
-#else
+#elif __linux__
         const char * swapoff_args[] = { "/sbin/swapoff", "-a", nullptr };
+#else
+#error "Swap-off command not known for this system"
 #endif
         run_process(swapoff_args, loop, sub_buf);
     }
