@@ -1,6 +1,8 @@
 #include <string>
 #include <iostream>
 #include <sstream>
+#include <vector>
+
 #include <cassert>
 #include <cstdlib>
 #include <cstring>
@@ -14,9 +16,48 @@ std::string test_service_dir;
 environment tenv;
 environment::env_map tenvmap;
 
+static std::vector<char> read_file_contents(const std::string &file_name)
+{
+    std::vector<char> contents;
+
+    int fd = open(file_name.c_str(), O_RDONLY);
+    if (fd == -1) {
+        throw std::system_error(errno, std::generic_category(), "read_file_contents: open");
+    }
+
+    char buf[1024];
+    int r = read(fd, buf, 1024);
+    while (r > 0) {
+        contents.insert(contents.end(), &buf[0], &buf[r]);
+        r = read(fd, buf, 1024);
+    }
+
+    if (r == -1) {
+        throw std::system_error(errno, std::generic_category(), "read_file_contents: read");
+    }
+
+    return contents;
+}
+
 void init_test_service_dir()
 {
     test_service_dir = "./test-services";
+
+    auto supply_service_file = [](const std::string &filename){
+        bp_sys::supply_file_content(filename, read_file_contents(filename));
+    };
+    supply_service_file("./test-services/t1");
+    supply_service_file("./test-services/t2");
+    supply_service_file("./test-services/t3");
+    supply_service_file("./test-services/t4");
+    supply_service_file("./test-services/fragment-1");
+    supply_service_file("./test-services/fragment-2");
+    supply_service_file("./test-services/dirtest1");
+    supply_service_file("./test-services/dirtest2");
+    supply_service_file("./test-services/dirtest3");
+    supply_service_file("./test-services/d1/d2/dirtest2");
+    supply_service_file("./test-services/d1/d2/dirtest3");
+
     tenvmap = tenv.build(main_env);
 }
 
@@ -61,7 +102,7 @@ void test_env_subst()
 
 void test_env_subst2()
 {
-    auto resolve_env_var = [](const std::string &name, environment::env_map const &) {
+    auto resolve_env_var = [](const std::string &name) {
         if (name == "ONE_VAR") return "a";
         if (name == "TWOVAR") return "hellohello";
         return "";
@@ -77,7 +118,7 @@ void test_env_subst2()
 
     dinit_load::read_setting_value(fpr, li, le, &offsets);
 
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test xa-a~ y${TWOVAR}hellohello$ONE_VAR");
 
@@ -89,7 +130,7 @@ void test_env_subst2()
 
 void test_env_subst3()
 {
-    auto resolve_env_var = [](const std::string &name, environment::env_map const &) {
+    auto resolve_env_var = [](const std::string &name) {
         if (name == "EMPTY") return "";
         if (name == "WS") return "    ";
         if (name == "PADDED") return " p ";
@@ -104,7 +145,7 @@ void test_env_subst3()
     std::string::iterator li = line.begin();
     std::string::iterator le = line.end();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     auto check_arg = [&](unsigned idx, const char *val)
     {
@@ -118,7 +159,7 @@ void test_env_subst3()
     line = "test $EMPTY foo";
     li = line.begin(); le = line.end(); offsets.clear();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test  foo");
     check_arg(1, "");
@@ -128,7 +169,7 @@ void test_env_subst3()
     line = "test $/EMPTY$/EMPTY$/EMPTY foo";
     li = line.begin(); le = line.end(); offsets.clear();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test  foo");
     check_arg(1, "foo");
@@ -137,7 +178,7 @@ void test_env_subst3()
     line = "test $/EMPTY$EMPTY$/EMPTY foo";
     li = line.begin(); le = line.end(); offsets.clear();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test  foo");
     check_arg(1, "");
@@ -147,7 +188,7 @@ void test_env_subst3()
     line = "test abc$/{EMPTY}def";
     li = line.begin(); le = line.end(); offsets.clear();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test abcdef");
     check_arg(1, "abcdef");
@@ -156,7 +197,7 @@ void test_env_subst3()
     line = "test abc$/{WS}def";
     li = line.begin(); le = line.end(); offsets.clear();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test abc def");
     check_arg(1, "abc");
@@ -166,7 +207,7 @@ void test_env_subst3()
     line = "test abc$/{PADDED}def";
     li = line.begin(); le = line.end(); offsets.clear();
     dinit_load::read_setting_value(fpr, li, le, &offsets);
-    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, &tenvmap, nullptr);
+    dinit_load::value_var_subst("command", line, offsets, resolve_env_var, nullptr);
 
     assert(line == "test abc p def");
     check_arg(1, "abc");
@@ -219,31 +260,39 @@ void test_settings()
             "rlimit-data = -:-";
 
     file_input_stack input_stack;
-    input_stack.add_source(ss.str(), "dummy");
+    bp_sys::supply_file_content("./dummy", ss.str());
+    dio::istream infile;
+    infile.open("./dummy");
+    input_stack.push("./dummy", std::move(infile), bp_sys::open(".", O_DIRECTORY));
 
     try {
+        auto resolve_var = [](const std::string &name) {
+            return (char *)nullptr;
+        };
+
         process_service_file("test-service", input_stack,
                 [&](string &line, file_pos_ref input_pos, string &setting,
                         dinit_load::setting_op_t op, string_iterator &i,
                         string_iterator &end) -> void {
 
-            auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist, const std::string &waitsford,
-                    dependency_type dep_type) -> void {
-                //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
-            };
+                    auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist,
+                            const std::string &waitsford, dependency_type dep_type) -> void {
+                        //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
+                    };
 
-            auto load_service_n = [&](const string &dep_name) -> const string & {
-                return dep_name;
-            };
+                    auto load_service_n = [&](const string &dep_name) -> const string & {
+                        return dep_name;
+                    };
 
-            try {
-                process_service_line(settings, "test-service", nullptr, line, input_pos, setting,
-                        op, i, end, load_service_n, process_dep_dir_n);
-            }
-            catch (service_description_exc &exc) {
-                //report_service_description_exc(exc);
-            }
-        });
+                    try {
+                        process_service_line(settings, "test-service", nullptr, line, input_pos,
+                                setting, op, i, end, load_service_n, process_dep_dir_n);
+                    }
+                    catch (service_description_exc &exc) {
+                        //report_service_description_exc(exc);
+                    }
+                },
+                nullptr, resolve_var);
     }
     catch (std::system_error &sys_err)
     {
@@ -290,30 +339,39 @@ void test_path_env_subst()
             "logfile = /some/$1/$username/${1}/dir\n";
 
     file_input_stack input_stack;
-    input_stack.add_source(ss.str(), "dummy");
+    bp_sys::supply_file_content("./dummy", ss.str());
+    dio::istream infile;
+    infile.open("./dummy");
+    input_stack.push("./dummy", std::move(infile), bp_sys::open(".", O_DIRECTORY));
 
     try {
+        auto resolve_var = [](const std::string &name) {
+            return (char *)nullptr;
+        };
+
         process_service_file("test-service", input_stack,
                 [&](string &line, file_pos_ref input_pos, string &setting,
-                        dinit_load::setting_op_t op, string_iterator &i, string_iterator &end) -> void {
+                        dinit_load::setting_op_t op, string_iterator &i,
+                        string_iterator &end) -> void {
 
-            auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist, const std::string &waitsford,
-                    dependency_type dep_type) -> void {
-                //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
-            };
+                    auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist,
+                            const std::string &waitsford, dependency_type dep_type) -> void {
+                        //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
+                    };
 
-            auto load_service_n = [&](const string &dep_name) -> const string & {
-                return dep_name;
-            };
+                    auto load_service_n = [&](const string &dep_name) -> const string & {
+                        return dep_name;
+                    };
 
-            try {
-                process_service_line(settings, "test-service", nullptr, line, input_pos, setting,
-                        op, i, end, load_service_n, process_dep_dir_n);
-            }
-            catch (service_description_exc &exc) {
-                //report_service_description_exc(exc);
-            }
-        });
+                    try {
+                        process_service_line(settings, "test-service", nullptr, line, input_pos,
+                                setting, op, i, end, load_service_n, process_dep_dir_n);
+                    }
+                    catch (service_description_exc &exc) {
+                        //report_service_description_exc(exc);
+                    }
+                },
+                nullptr, resolve_var);
     }
     catch (std::system_error &sys_err)
     {
@@ -366,24 +424,33 @@ void test_newline_err()
         dinit_load::service_settings_wrapper<prelim_dep> settings;
 
         file_input_stack input_stack;
-        input_stack.add_source(ss.str(), "dummy");
+        bp_sys::supply_file_content("./dummy", ss.str());
+        dio::istream infile;
+        infile.open("./dummy");
+        input_stack.push("./dummy", std::move(infile), bp_sys::open(".", O_DIRECTORY));
+
+        auto resolve_var = [](const std::string &name) {
+            return (char *)nullptr;
+        };
 
         process_service_file("test-service", input_stack,
                 [&](string &line, file_pos_ref input_pos, string &setting,
-                        dinit_load::setting_op_t op, string_iterator &i, string_iterator &end) -> void {
+                        dinit_load::setting_op_t op, string_iterator &i,
+                        string_iterator &end) -> void {
 
-            auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist, const std::string &waitsford,
-                    dependency_type dep_type) -> void {
-                //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
-            };
+                    auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist,
+                            const std::string &waitsford, dependency_type dep_type) -> void {
+                        //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
+                    };
 
-            auto load_service_n = [&](const string &dep_name) -> const string & {
-                return dep_name;
-            };
+                    auto load_service_n = [&](const string &dep_name) -> const string & {
+                        return dep_name;
+                    };
 
-            process_service_line(settings, "test-service", nullptr, line, input_pos, setting,
-                    op, i, end, load_service_n, process_dep_dir_n);
-        });
+                    process_service_line(settings, "test-service", nullptr, line, input_pos,
+                            setting, op, i, end, load_service_n, process_dep_dir_n);
+                },
+                nullptr, resolve_var);
     };
 
     try {
@@ -430,31 +497,40 @@ void test_newline2()
             " next line\n";
 
     file_input_stack input_stack;
-    input_stack.add_source(ss.str(), "dummy");
+    bp_sys::supply_file_content("./dummy", ss.str());
+    dio::istream infile;
+    infile.open("./dummy");
+    input_stack.push("./dummy", std::move(infile), bp_sys::open(".", O_DIRECTORY));
 
     try {
+        auto resolve_var = [](const std::string &name) {
+            return (char *)nullptr;
+        };
+
         process_service_file("test-service", input_stack,
                 [&](string &line, file_pos_ref input_pos, string &setting,
                         dinit_load::setting_op_t op, string_iterator &i,
                         string_iterator &end) -> void {
 
-            auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist, const std::string &waitsford,
-                    dependency_type dep_type) -> void {
-                //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
-            };
+                    auto process_dep_dir_n = [&](std::list<prelim_dep> &deplist,
+                            const std::string &waitsford,
+                            dependency_type dep_type) -> void {
+                        //process_dep_dir(name.c_str(), service_filename, deplist, waitsford, dep_type);
+                    };
 
-            auto load_service_n = [&](const string &dep_name) -> const string & {
-                return dep_name;
-            };
+                    auto load_service_n = [&](const string &dep_name) -> const string & {
+                        return dep_name;
+                    };
 
-            try {
-                process_service_line(settings, "test-service", nullptr, line, input_pos,
-                        setting, op, i, end, load_service_n, process_dep_dir_n);
-            }
-            catch (service_description_exc &exc) {
-                //report_service_description_exc(exc);
-            }
-        });
+                    try {
+                        process_service_line(settings, "test-service", nullptr, line, input_pos,
+                                setting, op, i, end, load_service_n, process_dep_dir_n);
+                    }
+                    catch (service_description_exc &exc) {
+                        //report_service_description_exc(exc);
+                    }
+                },
+                nullptr, resolve_var);
     }
     catch (std::system_error &sys_err)
     {
@@ -554,6 +630,29 @@ void test_plusassign()
     assert(ppi->second == 17);
 }
 
+void test_includes()
+{
+    dirload_service_set sset(test_service_dir.c_str());
+    auto t1 = sset.load_service("t4");
+    assert(t1->get_name() == "t4");
+    assert(t1->get_type() == service_type_t::PROCESS);
+}
+
+void test_dir_env_subst()
+{
+    dirload_service_set sset(test_service_dir.c_str());
+    bp_sys::setenv("DIR", "./d1/", true);
+    
+    auto dirtest1 = sset.load_service("dirtest1");
+    assert(dirtest1->get_dependencies().size() == 2);
+    auto &dep1 = dirtest1->get_dependencies().front();
+    auto &dep2 = dirtest1->get_dependencies().back();
+    assert(dep1.dep_type == dependency_type::WAITS_FOR);
+    assert(dep2.dep_type == dependency_type::WAITS_FOR);
+    assert(dep1.get_to()->get_name() == "dirtest2" || dep2.get_to()->get_name() == "dirtest2");
+    assert(dep1.get_to()->get_name() == "dirtest3" || dep2.get_to()->get_name() == "dirtest3");
+}
+
 #define RUN_TEST(name, spacing) \
     std::cout << #name "..." spacing << std::flush; \
     name(); \
@@ -574,6 +673,8 @@ int main(int argc, char **argv)
     RUN_TEST(test_newline2, "             ");
     RUN_TEST(test_comments, "             ");
     RUN_TEST(test_plusassign, "           ");
+    RUN_TEST(test_includes, "             ");
+    RUN_TEST(test_dir_env_subst, "        ");
     bp_sys::clearenv();
     return 0;
 }
